#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import str
import boto3
import requests

from sentrylogger.sentry_wrapper import ecs_error_logger

from cloud_resources import constants as consts
from cloud_resources.sentry_client import sentry_client
from cloud_resources.error_handler import ClusterError


def run_docker_task(cluster_name, task_def, over_rides,
                    region_name):
    """
    Runs a docker task on a given cluster on this machine and returns the exit
    status code after it exits.

    Starts the given docker/task definition belonging to the given cluster,
    waits for the Docker to exit and returns the exitcode of the Docker.
    Each task has a docker attached to it, and the docker runs the relevant
    algorithm.

    Parameters
    ----------
    cluster_name: String
        Name of the cluster
    task_def: String
        The task definition associated with the docker to be run.
    over_rides: Dict
        A dictionary containing the override options for docker to be run.
    region_name: String
        Name of the region running the cluster

    Returns
    -------
    List: task_ids
        List of task IDs run by the task definition in the cluster
    """
    try:
        client = boto3.client('ecs', region_name=region_name)
        instance_arn = requests.get(consts.EC2_METADATA_URL)\
            .json().get(consts.CONTAINER_ARN).split("/")[1]
        response = client.start_task(cluster=cluster_name, taskDefinition=task_def,
                                     overrides=over_rides,
                                     containerInstances=[instance_arn])
    except Exception as e:
        raise ClusterError(str(e))
    task_ids = [task.get(consts.TASK_ARN) for task in
                response.get(consts.TASKS)]
    ecs_error_logger(sentry_client, consts.ECS_FAILURE,
                     response.get(consts.FAILURES))
    return task_ids


def get_task_status(task_ids, cluster_name, region_name):
    """
    Retrieve task status from Cluster (ECS) API.

    It is assumed that only one Docker/Task definition is present in task_ids
    (The tasks to be tracked for status). Hence, the response of tasks
    status descrition API will contain only one task. Thus, the first task
    response returned by the API is fetched.

    Parameters
    ----------
    task_ids: List
        List of task IDs. It is assumed that this list consists of only one ID
    cluster_name: String
        Name of the cluster running the task
    region_name: String
        Name of the region running cluster

    Returns
    -------
    Dict
        Response of the describe Docker/task API.
    """
    try:
        client = boto3.client('ecs', region_name=region_name)
        response = client.describe_tasks(cluster=cluster_name, tasks=task_ids)
    except Exception as e:
        raise ClusterError(str(e))
    ecs_error_logger(sentry_client, consts.TASK_ERR,
                     response.get(consts.FAILURES))
    return response.get(consts.TASKS)[0]


def change_instance_protection(protection_status, region_name):
    """
    Updates the instance protection settings for the specified instances.
    Instance protection controls whether Auto Scaling can terminate a
    particular instance when scaling in or not. This function gets the ASG
    (Auto Scaling Group) client, ASG details such as ASG name, and sets the
    instance protection to enabled/disabled for the instance_id of current ec2
    machine.

    Parameters
    ----------
    protection_status: Boolean
        Instance protection setting. True for enabling and False for disabling
        instance protection.

    Returns
    -------
    String: resp
        JSON/dict containing the meta-data of set_instance_protection.
    """
    try:
        instance_id = get_current_machine_id()
        asg_client = boto3.client('autoscaling', region_name=region_name)
        asg_instance_info = asg_client.describe_auto_scaling_instances(
            InstanceIds=[instance_id])
    except Exception as e:
        raise ClusterError(str(e))
    asg_name = asg_instance_info.get('AutoScalingInstances')[0].\
        get('AutoScalingGroupName')
    resp = asg_client.set_instance_protection(InstanceIds=[
        instance_id], AutoScalingGroupName=asg_name,
        ProtectedFromScaleIn=protection_status)
    return resp


def get_current_machine_id():
    """
    Returns the unique id of current cloud machine (EC2).

    Returns
    -------
    String
        Current machine/instance ID (EC2 isntance id)
    """
    try:
        response = requests.get(consts.EC2_INSTANCE_ID_URL)
    except (requests.exceptions.RequestException) as e:
        raise ClusterError("Unable to fetch instance_id of current EC2 machine"
                           + str(e))
    return str(response.text)


def execute_policy(region_name, autoscaling_grp_name, policy_name, metric_value):
    """
    Manually triggers policy for autoscaling group

    Parameters
    ----------
    region_name: string
        region for which you want to trigger autoscaling group
    autoscaling_grp_name: string
        name of autoscaling group to trigger
    policy_name: string
        name of the policy to trigger
    metric_value: string
        number which should be greater than BreachThreshold value
        to execute policy
    """
    try:
        client = boto3.client(consts.AUTOSCALING, region_name=region_name)
        client.execute_policy(AutoScalingGroupName=autoscaling_grp_name,
                            PolicyName=policy_name,
                            HonorCooldown=True,
                            MetricValue=metric_value,
                            BreachThreshold=0)
    except Exception as e:
        raise ClusterError(str(e))
